#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2003 Free Software Foundation
#
#
# FILE:
# GFObjects.py
#
# DESCRIPTION:
# The majority of the objects that make up a form
#
# NOTES:
#
# HISTORY:
#

from gnue.common.datasources.GDataSource import GDataSource
from gnue.common import events

############################################################
# GFDataSource
#

class GFDataSource (GDataSource):
  def __init__(self, parent):
    GDataSource.__init__(self, parent, 'GFDataSource')
    self._toplevelParent = 'GFForm'
    self._form = self.findParentOfType('GFForm')


  #
  # Hooks for record-level triggers
  #

  def _beforeCommitInsert(self, record):
    self._form.dispatchEvent('preInsert', record=record)
    self._form.dispatchEvent('preCommit', record=record)

  def _beforeCommitUpdate(self, record):
    self._form.dispatchEvent('preUpdate', record=record)
    self._form.dispatchEvent('preCommit', record=record)

  def _beforeCommitDelete(self, record):
    self._form.dispatchEvent('preDelete', record=record)
    self._form.dispatchEvent('preCommit', record=record)

  def _onModification(self, record):
    try:
      record.__firstModification
    except NameError:
      record.__firstModification = 1
      self._form.dispatchEvent('onRecordTouched', record=record)
    return 1

  def _onRecordLoaded(self, record):
    self._form.dispatchEvent('onQueryNewRecord', record=record)
    return 1





