#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2003 Free Software Foundation
#
# FILE:
# wx/ToolBar.py
#
# DESCRIPTION:
# A generic UIdriver base for common message-based GUI toolkits.
#
# NOTES:
#

import string, sys
from qt import *
from gnue.forms.uidrivers._commonGuiToolkit.ToolBar import ToolBar as _BaseToolBar

class ToolBar(_BaseToolBar):

  # Create the menu
  def init(self):
    self.toolbar = QToolBar("GNUe Toolbar", self.container)
    self.toolbar.show()
    return self.toolbar

  # Add a menu item (action)
  def addAction(self, name, userAction):
    iconloc = userAction.getIconLocation(size="24x24")
    grayloc = userAction.getIconLocation(size="24x24-gray")

    # Set the action icon if available
    if iconloc:
      try:
        # Some caching logic for faster second/third forms
        icon = _cachedIcons[iconloc]
      except KeyError:
        icon = QIconSet(QPixmap(iconloc))
        _cachedIcons[iconloc] = icon
        if grayloc:
          icon.setPixmap(QPixmap(grayloc), QIconSet.Normal, QIconSet.Disabled)
        else:
          print "** WARNING: No gray version of '%s' to toolbar; no icon" % userAction.event

    else:
      print "** WARNING: Cannot add '%s' to toolbar; no icon" % userAction.event
      return

    button = QToolButton(self.toolbar, name)
    button.setIconSet(icon)
    button.setTextLabel(name)
    if userAction.canToggle:
      button.setToggleButton(1)

    return button


  # Add a separator
  def addSeparator(self):
    self.toolbar.addSeparator()

  # Enable a menu item
  def enableItem(self, item):
    item.setEnabled(1)

  # Disable a menu item
  def disableItem(self, item):
    item.setEnabled(0)

  def startingItem(self, item):
    item.setOn(1)

  def endingItem(self, item):
    item.setOn(0)


_cachedIcons = {}
